@@ -1,5 +1,6 @@
 module Main (main) where
 
+import Example (exampleTests)
 import Test.Framework (defaultMain)
 import Text.RSS.Tests (rssTests)
 import Text.Atom.Tests (atomTests)
@@ -10,4 +11,5 @@ main = defaultMain
   [ rssTests
   , atomTests
   , feedUtilTests
+  , exampleTests
   ]
